package com;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.List;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.apache.fop.apps.Driver;
import org.apache.fop.apps.FOPException;
import org.apache.fop.apps.Options;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.InputSource;

public class ThreadRunner implements Runnable {

	private static final String ENCODING_UTF8 = "UTF-8";

	private static final ClassLoader LOADER = ThreadRunner.class.getClassLoader();

	private static final String MAIN_DIR = LOADER.getResource(".") + "com";

	private final Logger log = LoggerFactory.getLogger(getClass());

	private static String userConfig;

	private static long lastUpdateTime = System.currentTimeMillis();

	private static final long DEFAULT_UPDATE_TIME = 5 * 60 * 1000; // 5 min.

	private int threadCounter;

	public int getThreadCounter() {
		return threadCounter;
	}

	public void setThreadCounter(int threadCounter) {
		this.threadCounter = threadCounter;
	}

	public ThreadRunner() {
		super();
		this.threadCounter = 1;
	}

	public ThreadRunner(int threadCounter) {
		super();
		this.threadCounter = threadCounter;
	}

	public void run() {
		for (int i = 0; i < threadCounter; i++) {
			String threadName = Thread.currentThread().getName();

			File classesDir = new File(CatalogUtil.TARGET + "/classes");
			File targetDir = new File(CatalogUtil.TARGET);

			File xslFile = new File(classesDir, "name2fo.xsl");
			File xmlFile = new File(classesDir, "name.xml");

			try {
				FopUserConfig fopUserConfig = buildUserConfig(MAIN_DIR);

				StreamSource streamSource = new StreamSource(new FileReader(xslFile));
				Transformer trans = TransformerFactory.newInstance().newTransformer(streamSource);
				StringWriter myWriter = new StringWriter();
				trans.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
				trans.transform(new StreamSource(new FileReader(xmlFile)), new StreamResult(myWriter));
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				final InputSource inputSource = new InputSource(new StringReader(myWriter.toString()));
				inputSource.setEncoding(ENCODING_UTF8);
				Driver driver = new Driver(inputSource, out);

				if (fopUserConfig != null) {
					log.debug("Refreshing fop user config");
					Options options = new Options();
					final InputSource userConfigSource = new InputSource(fopUserConfig.getUserConfig());
					options.loadUserconfiguration(userConfigSource);
				}

				driver.setRenderer(Driver.RENDER_PDF);
				driver.run();

				byte[] content = out.toByteArray();

				File resultFile = new File(targetDir, threadName + "_file[" + i + "].pdf");

				FileOutputStream fos = new FileOutputStream(resultFile);
				fos.write(content);

			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (TransformerException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (FOPException e) {
				e.printStackTrace();
			} catch (URISyntaxException e) {
				e.printStackTrace();
			}
		}
	}

	private FopUserConfig buildUserConfig(String mainDir) throws URISyntaxException {
		Long fontsUpdateTime = DEFAULT_UPDATE_TIME;

		if ((System.currentTimeMillis() - (lastUpdateTime + fontsUpdateTime) < 0) && (userConfig != null)) {
			log.trace("Current userConfig:" + userConfig);
			return new FopUserConfig(userConfig, false);
		}

		StringBuffer fontsBuf = new StringBuffer();

		String fontsDirName = mainDir + "/fonts";
		File fontsDir = new File(new URI(fontsDirName));
		if (fontsDir.exists() && fontsDir.isDirectory()) {
			List fonts = Arrays.asList(fontsDir.list(new FontsFilter()));
			List xmls = Arrays.asList(fontsDir.list(new XmlFilter()));
			XMLBuilder xm = new XMLBuilder();
			xm.addStartTag("configuration");
			xm.addStartTag("fonts");
			for (int i = 0; i < fonts.size(); i++) {
				String s = (String) fonts.get(i);
				String fontName = s.substring(0, s.length() - 4);
				String xml = fontName + ".xml";
				if (xmls.contains(xml)) {
					xm.sb.append("<font metrics-file='").append(mainDir).append("/fonts/").append(xml).append("'  kerning='yes' embed-file='").append(
							mainDir).append("/fonts/").append(s).append("'>\n");
					if (xml.endsWith("b.xml")) {
						fontName = fontName.substring(0, fontName.length() - 1);
						xm.sb.append("  <font-triplet name='").append(fontName).append("' style='normal' weight='bold' ");
						fontsBuf.append(" ").append(fontName).append("(style=normal weight=bold)");
					} else if (xml.endsWith("bi.xml")) {
						fontName = fontName.substring(0, fontName.length() - 2);
						xm.sb.append("  <font-triplet name='").append(fontName).append("' style='italic' weight='bold' ");
						fontsBuf.append(" ").append(fontName).append("(style=italic weight=bold)");
					} else if (xml.endsWith("i.xml")) {
						fontName = fontName.substring(0, fontName.length() - 1);
						xm.sb.append("  <font-triplet name='").append(fontName).append("' style='italic' weight='normal' ");
						fontsBuf.append(" ").append(fontName).append("(style=italic weight=normal)");
					} else {
						xm.sb.append("  <font-triplet name='").append(fontName).append("' style='normal' weight='normal' ");
						fontsBuf.append(" ").append(fontName).append("(style=normal weight=normal)");
					}
					xm.sb.append("/>\n");
					xm.sb.append("</font>\n");
				}
			}
			xm.addEndTag("fonts");
			xm.addEndTag("configuration");
			log.info("Dodano fonty do konfiguracji : " + fontsBuf);
			lastUpdateTime = System.currentTimeMillis();
			userConfig = xm.getXML();

			log.trace("Refreshed userConfig:" + userConfig);
			return new FopUserConfig(xm.getXML(), true);
		}
		log.warn("Brak katalogu z fontami - " + fontsDirName + " !!!");
		return null;
	}
}
