/**
 * User: roch
 * Date: Jan 13, 2004
 * Time: 11:38:15 AM
 */
package com;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.apache.log4j.Logger;

public class XMLBuilder {
	private static final Logger log = Logger.getLogger(XMLBuilder.class);

	private static final Map<String, String> entities = new LinkedHashMap<String, String>();

	static {
		entities.put("&", "&amp;");
		entities.put("<", "&lt;");
		entities.put(">", "&gt;");
		entities.put("€", "&#8364;");
		entities.put("^", "&#x5e;");
		entities.put("\"", "&#x22;");
		entities.put("'", "&#39;");
	}

	private static final Map<String, String> entitesReverse = new LinkedHashMap<String, String>();

	static {
		entitesReverse.put("&lt;", "<");
		entitesReverse.put("&gt;", ">");
		entitesReverse.put("&#8364;", "€");
		entitesReverse.put("&#x5e;", "^");
		entitesReverse.put("&#x22;", "\"");
		entitesReverse.put("&quot;", "\"");
		entitesReverse.put("&#39;", "'");
		entitesReverse.put("&amp;", "&");
	}

	public StringBuffer sb;
	List<String> unclosedtags = new ArrayList<String>();
	boolean closedtags = true;
	boolean changeEntities;

	public XMLBuilder(StringBuffer sb) {
		this(sb, true);
	}

	public XMLBuilder(StringBuffer sb, boolean changeEntities) {
		this.sb = sb;
		this.changeEntities = changeEntities;
	}

	public XMLBuilder() {
		this(new StringBuffer(), true);
	}

	public XMLBuilder(boolean changeEntities) {
		this(new StringBuffer(), changeEntities);
	}

	public void addStartTag(String tagname) {
		sb.append("<").append(tagname).append(">");
		unclosedtags.add(tagname);
	}

	public void addEndTag(String tagname) {
		sb.append("</").append(tagname).append(">");
		String temp = null;
		for(String unclosedtag : unclosedtags) {
			String s = unclosedtag;
			if(s.equals(tagname))
				temp = s;
		}
		if(temp != null)
			unclosedtags.remove(temp);
	}

	public void addXMLTag(String encoding) {
		sb.append("<?xml version=\"1.0\" encoding=\"").append(encoding).append("\"?>");
		closedtags = !closedtags;
	}

	public void addTag(String tagname, String value) {
		sb.append("<").append(tagname).append(">").append(_quote(value)).append("</").append(tagname).append(">");
	}

	public void addStartTag(String tagname, String value, String[] attributenames, String[] attributevalues) {
		unclosedtags.add(tagname);
		if(attributenames.length == attributevalues.length) {
			sb.append("<").append(tagname);
			for(int i = 0; i < attributenames.length; i++) {
				sb.append(" ").append(attributenames[i]).append("=\"").append(_quote(attributevalues[i])).append("\"");
			}
			sb.append(">");
			sb.append(_quote(value));
		} else {
			log.error("Attribute names length is not the same as Attributes values length");
		}
	}

	public void addTag(String tagname, String value, String[] attributenames, String[] attributevalues) {
		if(attributenames.length == attributevalues.length) {
			sb.append("<").append(tagname);
			for(int i = 0; i < attributenames.length; i++) {
				sb.append(" ").append(attributenames[i]).append("=\"").append(_quote(attributevalues[i])).append("\"");
			}
			sb.append(">");
			sb.append(_quote(value));
			sb.append("</").append(tagname).append(">");
		} else {
			log.error("Attribute names length is not the same as Attributes values length");
		}
	}

	public void addTag(String tagName, String value, String attributeName, String attributeValue) {
		addTag(tagName, value, new String[]{attributeName}, new String[]{attributeValue});
	}

	public void addEmpyTag(String tagname) {
		sb.append("<").append(tagname).append("/>");
	}

	public void clear() {
		sb = new StringBuffer();
	}

	public String getXML() {

		if(unclosedtags.size() > 0)
			return "<error>Unclosed tag: " + unclosedtags.get(0) + "</error>";
		return sb.toString();
	}

	public StringBuffer getBufferXML() {

		if(unclosedtags.size() > 0)
			return new StringBuffer("<error>Unclosed tag: " + unclosedtags.get(0) + "</error>");
		return sb;
	}

	protected String _quote(String str) {
		if(changeEntities) {
			return quote(str);
		}
		return str;
	}

	public static String quote(String str) {
		if(str == null) {
			return str;
		}
		Set<String> set = entities.keySet();
		for(String ent : set) {
			String replacement = (String) entities.get(ent);
			int i = 0;
			while(i < str.length()) {
				i = str.indexOf(ent, i);
				if(i > -1) {
					str = str.substring(0, i) + replacement + str.substring(i + ent.length());
					i += replacement.length();
				} else {
					break;
				}
			}
		}
		return str;
	}

	public static String removeEntites(String str) {
		if(str == null) {
			return str;
		}
		for(String entity : entitesReverse.keySet()) {
			String replacement = entitesReverse.get(entity);
			int i;
			while((i = str.toLowerCase().indexOf(entity)) >= 0) {
				str = str.substring(0, i) + replacement + str.substring(i + entity.length());
			}
		}
		return str;
	}
}
